@@ -1,29 +1,14 @@
-;;; hbdata.el --- Hyperbole button attribute accessor methods.
-
-;; Copyright (C) 1991-1995, 2008 Free Software Foundation, Inc.
-;; Developed with support from Motorola Inc.
-
-;; Author: Bob Weiner, Brown U.
-;; Maintainer: Mats Lidell <matsl@contactor.se>
-;; Keywords: hypermedia
-
+;;; hbdata.el --- GNU Hyperbole button attribute accessor functions
+;;
+;; Author:       Bob Weiner
+;;
+;; Orig-Date:     2-Apr-91
+;;
+;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; See the "HY-COPY" file for license information.
+;;
 ;; This file is part of GNU Hyperbole.
 
-;; GNU Hyperbole is free software; you can redistribute it and/or
-;; modify it under the terms of the GNU General Public License as
-;; published by the Free Software Foundation; either version 3, or (at
-;; your option) any later version.
-
-;; GNU Hyperbole is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
-
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
-
 ;;; Commentary:
 ;;
 ;;  This module handles Hyperbole button data/attribute storage.  In
@@ -46,11 +31,11 @@
 ;;    
 ;;    "TO-DO"
 ;;    (Key            Placeholders  LinkType      <arg-list>             creator and modifier with times)
-;;    ("alt.mouse.el" nil nil       link-to-file  ("./ell/alt-mouse.el") "zzz@cs.brown.edu" "19911027:09:19:26" "zzz" "19911027:09:31:36")
+;;    ("alt.mouse.el" nil nil       link-to-file  ("./ell/alt-mouse.el") "zzz@gnu.org" "19991027:09:19:26" "zzz@gnu.org" "19991027:09:31:36")
 ;;
 ;;  which means:  button \<(alt.mouse.el)> found in file "TO-DO" in the current
 ;;  directory provides a link to the local file "./ell/alt-mouse.el".  It was
-;;  created and last modified by zzz@cs.brown.edu.
+;;  created and last modified by zzz@gnu.org.
 ;;
 ;;  All link entries that originate from the same source file are stored
 ;;  contiguously, one per line, in reverse order of creation.
@@ -60,20 +45,19 @@
 ;;  source files within it.
 
 ;;; Code:
-
-;;;
+;;; ************************************************************************
 ;;; Other required Elisp libraries
-;;;
+;;; ************************************************************************
 
 (require 'hbmap)
 
-;;;
+;;; ************************************************************************
 ;;; Public functions
-;;;
+;;; ************************************************************************
 
-;;;
+;;; ------------------------------------------------------------------------
 ;;; Button data accessor functions
-;;;
+;;; ------------------------------------------------------------------------
 (defun hbdata:action (hbdata)
   "[Hyp V2] Returns action overriding button's action type or nil."
   (nth 1 hbdata))
@@ -150,9 +134,9 @@ Nil is returned when button has not beened modified."
       (goto-char end))
     src-matches))
 
-;;;
+;;; ------------------------------------------------------------------------
 ;;; Button data operators
-;;;
+;;; ------------------------------------------------------------------------
 
 (defun hbdata:build (&optional mod-lbl-key but-sym)
   "Tries to construct button data from optional MOD-LBL-KEY and BUT-SYM.
@@ -175,9 +159,8 @@ BUT-SYM nil means use 'hbut:current'.  If successful, returns a cons of
 	      (progn
 		(setq creator     (hbdata:creator entry)
 		      create-time (hbdata:create-time entry)
-		      modifier    (let* ((user (user-login-name))
-					 (addr (concat user
-						       hyperb:host-domain)))
+		      modifier    (let* ((user (hypb:user-name))
+					 (addr hyperb:user-email))
 				    (if (equal creator addr)
 					user addr))
 		      mod-time    (htz:date-sortable-gmt)
@@ -222,7 +205,7 @@ BUT-SYM nil means use 'hbut:current'.  If successful, returns a cons of
 			       ;; news messages.
 			       (action:path-args-abs args mail-dir)
 			     args)))
-		 (or creator (concat (user-login-name) hyperb:host-domain))
+		 (or creator hyperb:user-email)
 		 (or create-time (htz:date-sortable-gmt))
 		 modifier
 		 mod-time)
@@ -235,7 +218,7 @@ Returns nil if no matching entry is found.
 A button data entry is a list of attribute values.  Use methods from
 class 'hbdata' to operate on the entry."
   (hbdata:apply-entry
-   (function (lambda () (read (current-buffer))))
+   (lambda () (read (current-buffer)))
    lbl-key key-src directory))
 
 (defun hbdata:instance-next (lbl-key)
@@ -255,11 +238,11 @@ nil if LBL-KEY is nil."
 1 if not repeated, nil if no instance.
 Takes arguments LBL-KEY, KEY-SRC and optional DIRECTORY."
   (hbdata:apply-entry
-   (function (lambda () 
-	       (if (looking-at "[0-9]+")
-		   (string-to-number (buffer-substring (match-beginning 0)
-						    (match-end 0)))
-		 1)))
+   (lambda () 
+     (if (looking-at "[0-9]+")
+	 (string-to-number (buffer-substring (match-beginning 0)
+					     (match-end 0)))
+       1))
    lbl-key key-src directory nil 'instance))
 
 (defun hbdata:delete-entry (lbl-key key-src &optional directory)
@@ -267,29 +250,28 @@ Takes arguments LBL-KEY, KEY-SRC and optional DIRECTORY."
 Returns entry deleted (a list of attribute values) or nil.
 Use methods from class 'hbdata' to operate on the entry."
   (hbdata:apply-entry
-   (function
-    (lambda ()
-      (prog1 (read (current-buffer))
-	(let ((empty-file-entry "[ \t\n]*\\(\^L\\|\\'\\)")
-	      (kill))
-	  (beginning-of-line)
-	  (hbdata:delete-entry-at-point)
-	  (if (looking-at empty-file-entry)
-	      (let ((end (point))
-		    (empty-hbdata-file "[ \t\n]*\\'"))
-		(forward-line -1)
-		(if (= (following-char) ?\")
-		    ;; Last button entry for filename, so del filename.
-		    (progn (forward-line -1) (delete-region (point) end)))
-		(save-excursion
-		  (goto-char (point-min))
-		  (if (looking-at empty-hbdata-file)
-		      (setq kill t)))
-		(if kill
-		    (let ((fname buffer-file-name))
-		      (erase-buffer) (save-buffer) (kill-buffer nil)
-		      (hbmap:dir-remove (file-name-directory fname))
-		      (call-process "rm" nil 0 nil "-f" fname)))))))))
+   (lambda ()
+     (prog1 (read (current-buffer))
+       (let ((empty-file-entry "[ \t\n\r]*\\(\^L\\|\\'\\)")
+	     (kill))
+	 (beginning-of-line)
+	 (hbdata:delete-entry-at-point)
+	 (if (looking-at empty-file-entry)
+	     (let ((end (point))
+		   (empty-hbdata-file "[ \t\n\r]*\\'"))
+	       (forward-line -1)
+	       (if (eq (following-char) ?\")
+		   ;; Last button entry for filename, so del filename.
+		   (progn (forward-line -1) (delete-region (point) end)))
+	       (save-excursion
+		 (goto-char (point-min))
+		 (if (looking-at empty-hbdata-file)
+		     (setq kill t)))
+	       (if kill
+		   (let ((fname buffer-file-name))
+		     (erase-buffer) (save-buffer) (kill-buffer nil)
+		     (hbmap:dir-remove (file-name-directory fname))
+		     (call-process "rm" nil 0 nil "-f" fname))))))))
    lbl-key key-src directory))
 
 (defun hbdata:delete-entry-at-point ()
@@ -304,10 +286,9 @@ entry.  Optional INSTANCE non-nil means search for any button instance matching
 but-key."
   (let ((pos-entry-cons
 	 (hbdata:apply-entry
-	  (function
-	   (lambda ()
-	     (beginning-of-line)
-	     (cons (point) (read (current-buffer)))))
+	  (lambda ()
+	    (beginning-of-line)
+	    (cons (point) (read (current-buffer))))
 	  but-key key-src directory 'create instance)))
     (hbdata:to-entry-buf key-src directory)
     (forward-line 1)
@@ -316,9 +297,9 @@ but-key."
 	  (goto-char (car pos-entry-cons))
 	  (cdr pos-entry-cons)))))
 
-;;;
+;;; ************************************************************************
 ;;; Private functions
-;;;
+;;; ************************************************************************
 
 (defun hbdata:apply-entry (function lbl-key key-src &optional directory
 			   create instance)
@@ -339,24 +320,24 @@ Returns value of evaluation when a matching entry is found or nil."
 		nil
 	      (set-buffer key-src)
 	      (cond ((hmail:editor-p)
-		     (setq end-func (function (lambda ()
-						(hmail:msg-narrow)))))
+		     (setq end-func (lambda ()
+				      (hmail:msg-narrow))))
 		    ((and (hmail:lister-p)
 			  (progn (rmail:summ-msg-to) (rmail:to)))
 		     (setq opoint (point)
 			   key-src (current-buffer)
-			   end-func (function (lambda ()
-						(hmail:msg-narrow)
-						(goto-char opoint)
-						(lmail:to)))))
+			   end-func (lambda ()
+				      (hmail:msg-narrow)
+				      (goto-char opoint)
+				      (lmail:to))))
 		    ((and (hnews:lister-p)
 			  (progn (rnews:summ-msg-to) (rnews:to)))
 		     (setq opoint (point)
 			   key-src (current-buffer)
-			   end-func (function (lambda ()
-						(hmail:msg-narrow)
-						(goto-char opoint)
-						(lnews:to)))))))
+			   end-func (lambda ()
+				      (hmail:msg-narrow)
+				      (goto-char opoint)
+				      (lnews:to))))))
 	    (setq found (hbdata:to-entry-buf key-src directory create)))
 	(if found
 	    (let ((case-fold-search t)
@@ -459,9 +440,9 @@ On failure, returns nil."
       )))
 
 
-;;;
+;;; ************************************************************************
 ;;; Private variables
-;;;
+;;; ************************************************************************
 
 (provide 'hbdata)
 
