@@ -1,29 +1,15 @@
-;;; hlvar.el --- Permits use of Hyperbole variables in local variable lists.
-
-;; Copyright (C) 1985-1995 Free Software Foundation, Inc.
-;; Developed with support from Motorola Inc.
-
-;; Author: Bob Weiner, Brown U.
-;; Maintainer: Mats Lidell <matsl@contactor.se>
-;; Keywords: extensions, hypermedia
-
+;;; hlvar.el --- GNU Hyperbole variables in local variable lists
+;;
+;; Author:       Bob Weiner
+;;
+;; Orig-Date:     4-Nov-91 at 00:26:06
+;;
+;; Copyright (C) 1995-2016  Free Software Foundation, Inc.
+;; See the "HY-COPY" file for license information.
+;;
 ;; This file is part of GNU Hyperbole.
-
-;; GNU Hyperbole is free software; you can redistribute it and/or
-;; modify it under the terms of the GNU General Public License as
-;; published by the Free Software Foundation; either version 3, or (at
-;; your option) any later version.
-
-;; GNU Hyperbole is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
-
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
-
+;; This file is a slight variant of that found in "files.el" from GNU Emacs.
+;;
 ;;; Commentary:
 ;;
 ;;   Hyperbole uses the colon character extensively in its variable names.
@@ -37,13 +23,11 @@
 ;;
 ;;   So functionality is gained and none is lost, but a slight incompatibility
 ;;   in protocol is introduced.
-;;
 
 ;;; Code:
-
-;;;
+;;; ************************************************************************
 ;;; Public functions
-;;;
+;;; ************************************************************************
 
 (defun hack-local-variables (&optional force)
   "Parse, and bind or evaluate as appropriate, any local variables
@@ -98,7 +82,7 @@ for current buffer."
 	    (if suffix (setq suffix (concat (regexp-quote suffix) "$")))
 	    (while continue
 	      ;; Look at next local variable spec.
-	      (if selective-display (re-search-forward "[\n\C-m]")
+	      (if selective-display (re-search-forward "[\n\r]")
 		(forward-line 1))
 	      ;; Skip the prefix, if any.
 	      (if prefix
@@ -111,7 +95,7 @@ for current buffer."
 	      ;;
 	      ;; Bob Weiner - changed here to allow colons in var names.
 	      ;;
-	      (skip-chars-forward "^ \t\n")
+	      (skip-chars-forward "^ \t\n\r")
 	      (skip-chars-backward ":")
 	      (or (looking-at "[ \t]*:")
 		  (error "(hack-local-variables): Missing colon in local variables entry"))
@@ -128,7 +112,7 @@ for current buffer."
 		  (skip-chars-forward "^:")
 		  (forward-char 1)
 		  (setq val (read (current-buffer)))
-		  (skip-chars-backward "\n")
+		  (skip-chars-backward "\n\r")
 		  (skip-chars-forward " \t")
 		  (or (if suffix (looking-at suffix) (eolp))
 		      (error "Local variables entry is terminated incorrectly"))
@@ -139,7 +123,7 @@ for current buffer."
 			   (funcall (intern (concat (downcase (symbol-name val))
 						    "-mode"))))
 			  ((eq var 'eval)
-			   (if (string= (user-login-name) "root")
+			   (if (string-equal (user-login-name) "root")
 			       (message
 				"Ignoring `eval:' in file's local variables")
 			     (eval val)))
@@ -147,9 +131,7 @@ for current buffer."
 			     (set var val))))))))))
     (run-hooks 'hack-local-variables-hook)))
 
- 
-;; A new page avoids looking longer backwards for Local Variables
-;; section which gets confusing in this file.
+
 
 (provide 'hlvar)
 
