@@ -1,126 +1,308 @@
 ;;; hui-window.el --- Smart Mouse Key window and modeline depress/release actions.
-
-;; Copyright (C) 1992-1995, 2006 Free Software Foundation, Inc.
-;; Developed with support from Motorola Inc.
-
-;; Author: Bob Weiner, Brown U.
-;; Maintainer: Mats Lidell <matsl@contactor.se>
-;; Keywords: hypermedia, mouse
-
+;;
+;; Author:       Bob Weiner
+;;
+;; Orig-Date:    21-Sep-92
+;;
+;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; See the "HY-COPY" file for license information.
+;;
 ;; This file is part of GNU Hyperbole.
 
-;; GNU Hyperbole is free software; you can redistribute it and/or
-;; modify it under the terms of the GNU General Public License as
-;; published by the Free Software Foundation; either version 3, or (at
-;; your option) any later version.
-
-;; GNU Hyperbole is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
-
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
-
 ;;; Commentary:
 ;;
-;;   Must be loaded AFTER hmouse-alist has been defined in
-;;   "hui-mouse.el".
+;;   Must be loaded AFTER hmouse-alist has been defined in "hui-mouse.el".
 ;;
 ;;   Handles drags in same window or across windows and modeline depresses.
 ;;
-;; What drags and modeline presses do.
+;; What drags and modeline presses do.  (Note that a `thing' is a
+;; delimited expression, such as a string, list or markup language tag
+;; pair).
 ;; ==============================================================================
 ;;                                              Smart Keys
 ;; Context                         Action Key                 Assist Key
 ;; ==============================================================================
-;; Drag horizontally within window
-;;     Left to right               Scroll to buffer end       Split window across
-;;     Right to left               Scroll to buffer begin     Delete window
+;; Drag from thing start or end    Yank thing at release      Kill thing and yank at release
+;; Drag from shared window side
+;;   or from left of scroll bar    Resize window width        <- same
+;; Modeline depress & wind release Resize window height       <- same
 ;; Click in modeline
 ;;     Left window edge            Bury buffer                Unbury bottom buffer
 ;;     Right window edge           Info                       Smart Key Summary
-;;     Otherwise                   Action Key Hook            Assist Key Hook
-;; Modeline depress & wind release Resize window height       <- same
-;; Drag from shared window side    Resize window's width      <- same
-;; Drag from one window to another Create/modify a link but   Swap buffers
-;; Drag vertically within window   Split window sideways      <- same
-;; Drag diagonally within window   Save ring frame-config     Restore ring config
-;;
+;;     Otherwise                   Action Key Modeline Hook   Assist Key Modeline Hook
+;; Drag between windows            Create/modify a link but   Swap window buffers
+;; Drag in a window, region active Error, not allowed         Error, not allowed
+;; Drag horizontally in a window   Split window below         Delete window
+;; Drag vertically in a window     Split window side-by-side  Delete window
+;; Drag diagonally in a window     Save window-config         Restore window-config from ring
+;; Active region exists            Yank region at release     Kill region and yank at release
 
 ;;; Code:
+;;; ************************************************************************
+;;; Other required Elisp libraries
+;;; ************************************************************************
 
-;;;
+(eval-when-compile (defvar assist-flag nil)) ;; Silences free variable compiler warnings
+
+;;; ************************************************************************
 ;;; Public variables
-;;;
+;;; ************************************************************************
 
-(defvar action-key-modeline-hook 'hmouse-context-menu
-  "A list of functions to call when the Action Mouse Key is clicked in the center portion of a modeline.")
+(defcustom action-key-minibuffer-function #'hyperbole
+  "*Function run by the Action Key after a click in an inactive minibuffer.
+Its default value is #'hyperbole, which displays the Hyperbole minibuffer menu."
+  :type 'function
+  :group 'hyperbole-keys)
 
-(defvar assist-key-modeline-hook nil
-  "A list of functions to call when the Assist Mouse Key is clicked in the center portion of a modeline.")
+(defcustom assist-key-minibuffer-function #'hui-menu-screen-commands
+  "*Function run by the Assist Key after a click in an inactive minibuffer.
+Its default value is #'hui-menu-screen-commands, which displays a popup menu
+of screen control commands."
+  :type 'function
+  :group 'hyperbole-keys)
 
-(defvar hmouse-edge-sensitivity 10
+(defcustom action-key-modeline-function #'hmouse-context-menu
+  "Function to call when the Action Mouse Key is clicked in the center portion of a modeline."
+  :type 'function
+  :group 'hyperbole-keys)
+
+(defcustom assist-key-modeline-function #'hui-menu-screen-commands
+  "Function to call when the Assist Mouse Key is clicked in the center portion of a modeline."
+  :type 'function
+  :group 'hyperbole-keys)
+
+ ;; Mats Lidell says this should be 10 characters for GNU Emacs.
+(defvar hmouse-edge-sensitivity (if hyperb:emacs-p 10 3)
   "*Number of characters from window edges within which a click is considered at an edge.")
 
-(defvar hmouse-side-sensitivity (if hyperb:emacs19-p 2 1)
+(defvar hmouse-side-sensitivity (if hyperb:emacs-p 5 1)
   "*Characters in either direction from window side within which a click is considered on the side.")
 
 (defvar hmouse-x-drag-sensitivity 5
-  "*Number of chars mouse must move horizontally between depress/release to register a horizontal drag.")
+  "*Number of characters mouse must move horizontally between depress/release to register a horizontal drag.")
 
 (defvar hmouse-y-drag-sensitivity 3
   "*Number of lines mouse must move vertically between depress/release to register a vertical drag.")
 
 (defvar hmouse-x-diagonal-sensitivity 4
-  "*Number of chars mouse must move horizontally between depress/release to register a diagonal drag.")
+  "*Number of characters mouse must move horizontally between depress/release to register a diagonal drag.")
 (defvar hmouse-y-diagonal-sensitivity 3
   "*Number of lines mouse must move vertically between depress/release to register a diagonal drag.")
 
 ;;;
-;;; Add mode line handling to hmouse-alist dispatch table.
+;;; Add window handling to hmouse-alist dispatch table.
 ;;;
 (if (not (boundp 'hmouse-alist))
     (error
-      "\"hui-modeln.el\": hmouse-alist must be defined before loading this.")
-  (or (memq 'hmouse-drag-window-side
-	    (mapcar (function (lambda (elt) (let ((pred (car elt)))
-					      (if (listp pred) (car pred)))))
-		    hmouse-alist))
+      "\"hui-window.el\": `hmouse-alist' must be defined before loading this.")
+  (or (assoc #'(hmouse-drag-thing) hmouse-alist)
       (setq hmouse-alist
 	    (append
 	      '(
+		;; If click in the minibuffer when it is not active (blank),
+		;; display the Hyperbole minibuffer menu or popup the jump menu.
+		((hmouse-inactive-minibuffer-p) .
+		 ((funcall action-key-minibuffer-function) . 
+		  (funcall assist-key-minibuffer-function)))
+		((hmouse-drag-thing) .
+		 ((hmouse-yank-region) . (hmouse-kill-and-yank-region)))
 		((hmouse-drag-window-side) .
-		 ((hmouse-resize-window-side) .
-		  (hmouse-resize-window-side 'assist)))
-		((setq hkey-value 
-		       (and (not (hmouse-drag-between-windows))
-			    (hmouse-drag-horizontally))) .
-		 ((hmouse-horizontal) . (hmouse-horizontal-assist)))
+		 ((hmouse-resize-window-side) . (hmouse-resize-window-side 'assist)))
 		((hmouse-modeline-depress) .
 		 ((action-key-modeline) . (assist-key-modeline)))
 		((hmouse-drag-between-windows) .
+		 ;; Note that `hui:link-directly' uses any active
+		 ;; region as the label of the button to create.
 		 ((hui:link-directly) . (hmouse-swap-buffers 'assist)))
+		((hmouse-drag-region-active) .
+		 ((hmouse-drag-not-allowed) . (hmouse-drag-not-allowed)))
+		((setq hkey-value (and (not (hmouse-drag-between-windows))
+				       (hmouse-drag-horizontally))) .
+		 ((hmouse-horizontal-action-drag) . (hmouse-horizontal-assist-drag)))
 		((hmouse-drag-vertically) .
-		 ((sm-split-window-horizontally) .
-		  (sm-split-window-horizontally)))
+		 ((hmouse-vertical-action-drag) . (hmouse-vertical-assist-drag)))
 		((setq hkey-value (hmouse-drag-diagonally)) .
-		 ((wconfig-ring-save) .
-		  (wconfig-yank-pop
-		    (prefix-numeric-value current-prefix-arg))))
+		 ((hywconfig-ring-save) . (hywconfig-yank-pop
+					 (prefix-numeric-value current-prefix-arg))))
+		;;
+		;; Now since this is not a drag and if there was an active
+		;; region prior to when the Action or Assist Key was
+		;; pressed, then store point at one end of the region into
+		;; `hkey-value' and the string value of the region
+		;; into `hkey-region' which is either yanked, or
+		;; killed and yanked at the current point.
+		((hmouse-prior-active-region) .
+		 ((hmouse-yank-region) . (hmouse-kill-and-yank-region)))
+		;;
 		)
 	      hmouse-alist))))
 
-
-;;;
+;;; ************************************************************************
 ;;; Public functions
-;;;
+;;; ************************************************************************
+
+(defun hmouse-context-menu ()
+  "If running under a window system, display or hide the buffer menu.
+If not running under a window system and Smart Menus are loaded, display the
+appropriate Smart Menu for the context at point.  (Smart Menus are a
+part of InfoDock and not a part of Hyperbole)."
+  (interactive)
+  (if (and (fboundp 'smart-menu)
+	   (or (null window-system)
+	       (not (or hyperb:emacs-p (featurep 'xemacs)))))
+      (smart-menu)
+    (let ((wind (get-buffer-window "*Buffer List*"))
+	  owind)
+      (if wind
+	  (unwind-protect
+	      (progn (setq owind (selected-window))
+		     (select-window wind)
+		     (bury-buffer nil))
+	    (select-window owind))
+	(buffer-menu)))))
+
+(defun hmouse-context-ibuffer-menu ()
+  "If running under a window system, display or hide the IBuffer menu.
+If not running under a window system and Smart Menus are loaded, display the
+appropriate Smart Menu for the context at point.  (Smart Menus are a
+part of InfoDock and not a part of Hyperbole)."
+  (interactive)
+  (if (and (fboundp 'smart-menu)
+	   (or (null window-system)
+	       (not (or hyperb:emacs-p (featurep 'xemacs)))))
+      (smart-menu)
+    (let ((wind (get-buffer-window "*Ibuffer*"))
+	  owind)
+      (if wind
+	  (unwind-protect
+	      (progn (setq owind (selected-window))
+		     (select-window wind)
+		     (bury-buffer nil))
+	    (select-window owind))
+	(ibuffer)))))
+
+(defun hmouse-prior-active-region ()
+  (and (region-active-p)
+       ;; Store and goto any prior value of point from the region
+       ;; prior to the Smart Key depress, so we can return to it later.
+       (setq hkey-value (marker-position
+			 (if assist-flag assist-key-depress-prev-point action-key-depress-prev-point)))
+       (goto-char hkey-value)
+       (hmouse-save-region)))
+
+(defun hmouse-drag-region-active ()
+  "Return non-nil if an active region existed in the depress buffer prior to the depress and a drag motion has occurred."
+  (save-excursion
+    (hmouse-goto-depress-prev-point)
+    (and (region-active-p)
+	 (or (hmouse-drag-vertically) (hmouse-drag-horizontally) (hmouse-drag-diagonally))
+	 (setq hkey-value (point)))))
+
+(defun hmouse-drag-thing ()
+  "Return t if no region is active and a Smart Key drag began at the start/end of a delimited construct and ended at some other point in the same buffer, else nil.
+Delimited constructs include lists, comments, strings,
+ arrays/vectors, sets, and markup pair tags, such as <div>
+ </div>.  Point must be on the start or end delimiter or in the
+ case of markup pair tags, on the first character of either tag.
+ For strings and comments, point must be on the first line."
+    ;; Move point back to Smart Key depress location for testing whether at a thing.
+    (let ((depress-args (if assist-flag assist-key-depress-args action-key-depress-args))
+	  (release-args (if assist-flag assist-key-release-args action-key-release-args))
+	  (marked-thing))
+      (save-excursion
+	(hmouse-goto-depress-point)
+	(if (and (not (region-active-p)) (hui-select-at-delimited-thing-p)
+		 (or (markerp depress-args) (markerp release-args)
+		     (and (not (or (hmouse-drag-window-side) (hmouse-modeline-depress)))
+			  (or (hmouse-drag-between-windows) (hmouse-drag-vertically) 
+			      (hmouse-drag-horizontally) (hmouse-drag-diagonally))))
+		 (let ((start-buf (window-buffer (smart-window-of-coords depress-args)))
+		       (end-buf (window-buffer (smart-window-of-coords release-args)))
+		       (start-point (smart-point-of-coords depress-args))
+		       (end-point (smart-point-of-coords release-args)))
+		   ;; Ignore this, if it is a click or if drag end point is within the thing to operate upon
+		   (not (and (eq start-buf end-buf)
+			     (/= start-point end-point)
+			     (setq marked-thing (hui-select-delimited-thing))
+			     (>= end-point (min (point) (mark)))
+			     (<= end-point (max (point) (mark)))))))
+	    (progn (if (not (region-active-p)) (hui-select-delimited-thing))
+		   ;; Store any new value of point as a result of marking the region,
+		   ;; so we can return to it later.
+		   (setq hkey-value (point))
+		   (hmouse-save-region)
+		   t)
+	  (if marked-thing (deactivate-mark))
+	  nil))))
+
+(defun hmouse-kill-region ()
+  "Kill the marked region near where the Smart Key was depressed.
+Signals an error if the depress buffer is read-only."
+  ;; Move point back to Smart Key depress buffer but not necessarily
+  ;; the same point at the depress since region selection may have
+  ;; moved it.
+  (hmouse-goto-region-point)
+  (if buffer-read-only
+      ;; In this case, we want an error that will terminate execution so that
+      ;; hkey-region is not reset to nil.  This allows the user to fix the
+      ;; problem and then to try killing again.
+      (error "(hmouse-kill-region): Use {%s} to enable killing from this buffer."
+	     (hmouse-read-only-toggle-key))
+    (kill-region (point) (mark))))
+
+(defun hmouse-kill-and-yank-region ()
+  "Kill the marked region near where the Smart Key was depressed and yank it at the point of release.
+Signals an error if either depress or release buffer is read-only."
+  ;; Move point back to Smart Key depress buffer but not necessarily
+  ;; the same point, for testing if real-only.
+  (hmouse-goto-region-point)
+  (if buffer-read-only
+      ;; In this case, we want an error that will terminate execution so that
+      ;; hkey-region is not reset to nil.  This allows the user to fix the
+      ;; problem and then to try killing again.
+      (error "(hmouse-kill-and-yank-region): Use {%s} to enable killing from this buffer."
+	     (hmouse-read-only-toggle-key))
+    ;; Depress and release may be in the same buffer, in which case, 
+    ;; save the release point that may change as a result of
+    ;; the kill; also, before the kill, restore the point to where it
+    ;; was when the region was set.
+    (hmouse-goto-release-point)
+    (let ((release-point (point-marker)))
+      (if buffer-read-only
+	  ;; In this case, we want an error that will terminate execution so that
+	  ;; hkey-region is not reset to nil.  This allows the user to fix the
+	  ;; problem and then to try yanking again.
+	  (error "(hmouse-kill-and-yank-region): Use {%s} to enable yanking into this buffer."
+		 (hmouse-read-only-toggle-key))
+	;; Now kill and yank the region into the Smart Key release buffer.
+	(hmouse-goto-region-point)
+	(kill-region (point) (mark))
+	;; Permanently return to release point
+	(select-window (if assist-flag assist-key-release-window action-key-release-window))
+	(goto-char release-point)
+	(hmouse-insert-region)))))
+
+(defun hmouse-yank-region ()
+  "Yank the region of text saved in `hkey-region' into the current buffer.
+Signals an error if the buffer is read-only."
+  ;; If a region was just selected for yanking, deactivate it as we
+  ;; have already copied the region into `hkey-region'.
+  (hmouse-goto-region-point)
+  (if (region-active-p) (deactivate-mark))
+  (hmouse-goto-release-point)
+  (if buffer-read-only
+      ;; In this case, we want an error that will terminate execution so that
+      ;; hkey-region is not reset to nil.  This allows the user to fix the
+      ;; problem and then to try yanking again.
+      (error "(hmouse-yank-region): Use {%s} to enable yanking into this buffer."
+	     (hmouse-read-only-toggle-key))
+    ;; Permanently return to release point
+    (select-window (if assist-flag assist-key-release-window action-key-release-window))
+    (hmouse-insert-region)))
 
 (defun hmouse-drag-between-windows ()
   "Returns non-nil if last Action Key depress and release were in different windows.
-If free variable 'assist-flag' is non-nil, uses Assist Key."
+If free variable `assist-flag' is non-nil, uses Assist Key."
   (if assist-flag
       (and assist-key-depress-window assist-key-release-window
 	   (not (eq assist-key-depress-window
@@ -130,7 +312,7 @@ If free variable 'assist-flag' is non-nil, uses Assist Key."
 
 (defun hmouse-drag-diagonally ()
   "Returns non-nil iff last Action Key use was a diagonal drag within a single window.
-If free variable 'assist-flag' is non-nil, uses Assist Key.
+If free variable `assist-flag' is non-nil, uses Assist Key.
 Value returned is nil if not a diagonal drag, or one of the following symbols
 depending on the direction of the drag: southeast, southwest, northwest, northeast."
   (let ((last-depress-x) (last-release-x)
@@ -160,7 +342,7 @@ depending on the direction of the drag: southeast, southwest, northwest, northea
 
 (defun hmouse-drag-horizontally ()
   "Returns non-nil iff last Action Key use was a horizontal drag within a single window.
-If free variable 'assist-flag' is non-nil, uses Assist Key.
+If free variable `assist-flag' is non-nil, uses Assist Key.
 Value returned is nil if not a horizontal drag, 'left if drag moved left or
 'right otherwise."
   (let ((last-depress-x) (last-release-x)
@@ -187,7 +369,7 @@ Value returned is nil if not a horizontal drag, 'left if drag moved left or
 
 (defun hmouse-drag-vertically ()
   "Returns non-nil iff last Action Key use was a vertical drag within a single window.
-If free variable 'assist-flag' is non-nil, uses Assist Key.
+If free variable `assist-flag' is non-nil, uses Assist Key.
 Value returned is nil if not a vertical line drag, 'up if drag moved up or
 'down otherwise."
   (let ((last-depress-x) (last-release-x)
@@ -213,68 +395,76 @@ Value returned is nil if not a vertical line drag, 'up if drag moved up or
 	     hmouse-x-drag-sensitivity)
 	 (if (< last-depress-y last-release-y) 'down 'up))))
 
-(or (fboundp 'abs)
-    (defun abs (number)
-      "Return the absolute value of NUMBER."
-      (cond
-	((< number 0)
-	 (- 0 number))
-	(t number))))
-
 (defun hmouse-drag-window-side ()
   "Returns non-nil if Action Key was dragged from a window side divider.
-If free variable 'assist-flag' is non-nil, uses Assist Key."
-  (cond (hyperb:xemacs-p
+If free variable `assist-flag' is non-nil, uses Assist Key."
+  (cond ((featurep 'xemacs)
 	 ;; Depress events in scrollbars or in non-text area of buffer are
 	 ;; not visible or identifiable at the Lisp-level, so always return
 	 ;; nil.
 	 nil)
-	(hyperb:window-system
+	((hyperb:window-system)
 	 (let* ((depress-args (if assist-flag assist-key-depress-args
 				action-key-depress-args))
 		(release-args (if assist-flag assist-key-release-args
 				action-key-release-args))
 		(w (smart-window-of-coords depress-args))
-		(side-ln (and w (1- (nth 2 (window-edges w)))))
+		(right-side-ln (and w (1- (nth 2 (window-edges w)))))
 		(last-press-x   (hmouse-x-coord depress-args))
 		(last-release-x (hmouse-x-coord release-args)))
-	   (and last-press-x last-release-x side-ln
+	   (and last-press-x last-release-x right-side-ln
 		(/= last-press-x last-release-x)
-		(/= (1+ side-ln) (frame-width))
-		(<= (max (- last-press-x side-ln) (- side-ln last-press-x))
+		(not (<= (abs (- right-side-ln (frame-width))) 5))
+		(<= (abs (- last-press-x right-side-ln))
 		    hmouse-side-sensitivity))))))
 
-(defun sm-split-window-horizontally ()
-  "Splits current window in two evenly, side by side.
-Beeps and prints message if can't split window further."
-  (interactive)
-  (let ((window-min-width 5))
-    (condition-case ()
-	(split-window-horizontally nil)
-      (error (progn (beep)
-		    (message
-		     "(sm-split-window-horizontally): Can't split window further."))))))
+(defun hmouse-read-only-toggle-key ()
+  "Return the first key binding that toggles read-only mode, or nil if none."
+  (key-description (or (where-is-internal #'read-only-mode nil t)
+		       (where-is-internal #'vc-toggle-read-only nil t)
+		       (where-is-internal #'toggle-read-only nil t))))
 
-(defun sm-split-window-vertically ()
-  "Splits current window in two evenly, one above the other.
-Beeps and prints message if can't split window further."
+(defun hmouse-vertical-action-drag ()
+  "Handles an Action Key vertical drag within a window: adds a window to the right of this one.
+Beeps and prints message if the window cannot be split further."
   (interactive)
-  (let ((window-min-height 2))
-    (condition-case ()
-	(if (fboundp 'split-window-quietly)
-	    (split-window-quietly nil)
-	  (split-window-vertically nil))
-      (error
-	(progn
-	  (beep)
-	  (message
-	    "(sm-split-window-vertically): Can't split window further."))))))
+  (condition-case ()
+      (split-window-horizontally nil)
+    (error (beep)
+	   (message "(hmouse-vertical-action-drag): Can't split the window further."))))
+
+(defun hmouse-vertical-assist-drag ()
+  "Handles an Assist Key vertical drag within a window: deletes the current window.
+Beeps and prints message if the window cannot be split further."
+  (condition-case ()
+      (delete-window)
+    (error (beep)
+	   (message "(hmouse-vertical-assist-drag): A single window cannot be deleted."))))
+
+(defun hmouse-horizontal-action-drag ()
+  "Handles an Action Key vertical drag within a window: adds a window below this one.
+Beeps and prints message if the window cannot be split further."
+  (interactive)
+  (condition-case ()
+      (if (fboundp 'split-window-quietly)
+	  (split-window-quietly nil)
+	(split-window-vertically nil))
+    (error (beep)
+	   (message "(hmouse-horizontal-action-drag): Can't split the window further."))))
+
+(defun hmouse-horizontal-assist-drag ()
+  "Handles an Assist Key horizontal drag within a window: deletes the current window.
+Beeps and prints message if the window cannot be split further."
+  (condition-case ()
+      (delete-window)
+    (error (beep)
+	   (message "(hmouse-horizontal-assist-drag): A single window cannot be deleted."))))
 
 (defun smart-coords-in-window-p (coords window)
   "Tests if COORDS are in WINDOW.  Returns WINDOW if they are, nil otherwise."
-  (cond ((and hyperb:emacs19-p (eventp coords))
+  (cond ((and hyperb:emacs-p (eventp coords))
 	 (eq (posn-window (event-start coords)) window))
-	((if hyperb:xemacs-p
+	((if (featurep 'xemacs)
 	     (if (eventp coords)
 		 (eq (event-window coords) window)
 	       (eq (car coords) window))))
@@ -290,12 +480,24 @@ Beeps and prints message if can't split window further."
 		  (<= w-ymin y) (<= y w-ymax)
 		  window)))))
 
+(defun smart-point-of-coords (coords)
+  "Returns point within window in which COORDS fall or nil if none.
+Ignores minibuffer window."
+  (cond ((markerp coords)
+	 (marker-position coords))
+	((and hyperb:emacs-p (eventp coords))
+	 (posn-point (event-start coords)))
+	((and (featurep 'xemacs) (eventp coords))
+	 (event-point coords))))
+
 (defun smart-window-of-coords (coords)
   "Returns window in which COORDS fall or nil if none.
 Ignores minibuffer window."
-  (cond ((and hyperb:emacs19-p (eventp coords))
+  (cond ((markerp coords)
+	 (get-buffer-window (marker-buffer coords)))
+	((and hyperb:emacs-p (eventp coords))
 	 (posn-window (event-start coords)))
-	((if hyperb:xemacs-p
+	((if (featurep 'xemacs)
 	     (if (eventp coords)
 		 (event-window coords)
 	       (car coords))))
@@ -308,43 +510,66 @@ Ignores minibuffer window."
 		     window (smart-coords-in-window-p coords w)))
 	     window))))
 
-;;;
+;;; ************************************************************************
 ;;; Private functions
-;;;
+;;; ************************************************************************
 
-(defun hmouse-context-menu ()
-  "If running under a window system, display or hide the buffer menu.
-If not running under a window system and Smart Menus are loaded, display the
-appropriate Smart Menu for the context at point."
-  (if (and (fboundp 'smart-menu)
-	   (or (null window-system)
-	       (not (or hyperb:xemacs-p hyperb:emacs19-p))))
-      (smart-menu)
-    (let ((wind (get-buffer-window "*Buffer List*"))
-	  owind)
-      (if wind
-	  (unwind-protect
-	      (progn (setq owind (selected-window))
-		     (select-window wind)
-		     (bury-buffer nil))
-	    (select-window owind))
-	(buffer-menu nil)))))
+(defun hmouse-drag-not-allowed ()
+  "Display an error when a region is active and in-window drags are not allowed."
+  ;; Return point to where it was prior to depress so the region does not permanently change.
+  (goto-char hkey-value)
+  (error "(hmouse-drag-region-active): Region is active; Smart Key drags are not allowed.  Use a press/click instead."))
 
-(defun hmouse-horizontal ()
-  "Goes to buffer end if drag was to the right, otherwise goes to beginning."
-  (if (eq hkey-value 'right)
-      (end-of-buffer)
-    (beginning-of-buffer)))
+(defun hmouse-set-buffer-and-point (buffer point)
+  (when buffer
+    (set-buffer buffer)
+    (if point (goto-char point))))
 
-(defun hmouse-horizontal-assist ()
-  "Splits window vertically if drag was to the right, otherwise deletes window."
-  (if (eq hkey-value 'right)
-      (sm-split-window-vertically)
-    (delete-window)))
+(defun hmouse-goto-depress-prev-point ()
+  "Temporarily set point to where the last Smart Key was depressed."
+  (let ((buf (marker-buffer (if assist-flag assist-key-depress-prev-point action-key-depress-prev-point)))
+	(loc (marker-position (if assist-flag assist-key-depress-prev-point action-key-depress-prev-point))))
+    (hmouse-set-buffer-and-point buf loc)))
+
+(defun hmouse-goto-depress-point ()
+  "Temporarily set point to where the last Smart Key was depressed."
+  (let ((buf (window-buffer (if assist-flag assist-key-depress-window action-key-depress-window)))
+	(loc (smart-point-of-coords (if assist-flag assist-key-depress-args action-key-depress-args))))
+    (hmouse-set-buffer-and-point buf loc)))
+
+(defun hmouse-goto-region-point ()
+  "Temporarily set point back to where it was when the region was activated."
+  (let ((buf (window-buffer (if assist-flag assist-key-depress-window action-key-depress-window)))
+	(loc hkey-value))
+    (hmouse-set-buffer-and-point buf loc)))
+
+(defun hmouse-goto-release-point ()
+  "Temporarily set point to where the last Smart Key was depressed."
+  (let ((buf (window-buffer (if assist-flag assist-key-release-window action-key-release-window)))
+	(loc (smart-point-of-coords (if assist-flag assist-key-release-args action-key-release-args))))
+    (hmouse-set-buffer-and-point buf loc)))
+
+(defun hmouse-inactive-minibuffer-p ()
+  "Return t if the last command event was a mouse press or release within an inactive minibuffer, else nil."
+  (if (= (minibuffer-depth) 0)
+      (eq (minibuffer-window) (posn-window (event-start last-command-event)))))
+
+(defun hmouse-insert-region ()
+  "Save a mark, then insert at point the text from `hkey-region' and indent it."
+  (indent-for-tab-command)
+  (push-mark nil t)
+  (if (eobp) (insert "\n"))
+  (insert hkey-region)
+  (indent-region (point) (mark))
+  (message "") ;; Clear any indenting message.
+  ;; From par-align.el library; need to think about all the
+  ;; possibilities before enabling filling of this region.
+  ;; (if (fboundp 'fill-region-and-align) (fill-region-and-align (mark) (point)))
+  )
 
 (defun action-key-modeline ()
   "Handles Action Key depresses on a window mode line.
-If key is:
+If the Action Key is:
  (1) clicked on left edge of a window's modeline,
      window's buffer is buried (placed at bottom of buffer list);
  (2) clicked on right edge of a window's modeline,
@@ -352,7 +577,7 @@ If key is:
      modeline clicked belongs to a window displaying Info, the Info
      buffer is hidden;
  (3) clicked anywhere in the middle of a window's modeline,
-     the functions listed in 'action-key-modeline-hook' are called;
+     the function given by `action-key-modeline-function' is called;
  (4) dragged vertically from modeline to within a window,
      the modeline is moved to point of key release, thereby resizing
      its window and potentially its vertical neighbors."
@@ -364,12 +589,12 @@ If key is:
 		  (if (eq major-mode 'Info-mode)
 		      (Info-exit)
 		    (info)))
-		 (t (run-hooks 'action-key-modeline-hook))))
+		 (t (funcall action-key-modeline-function))))
 	  (t (hmouse-modeline-resize-window)))))
 
 (defun assist-key-modeline ()
   "Handles Assist Key depresses on a window mode line.
-If secondary key is:
+If the Assist Key is:
  (1) clicked on left edge of a window's modeline,
      bottom buffer in buffer list is unburied and placed in window;
  (2) clicked on right edge of a window's modeline,
@@ -377,7 +602,7 @@ If secondary key is:
      displayed and the modeline clicked belongs to a window displaying
      the summary, the summary buffer is hidden;
  (3) clicked anywhere in the middle of a window's modeline,
-     the functions listed in 'assist-key-modeline-hook' are called;
+     the function given by `assist-key-modeline-function' is called;
  (4) dragged vertically from modeline to within a window,
      the modeline is moved to point of key release, thereby resizing
      its window and potentially its vertical neighbors."
@@ -391,10 +616,10 @@ If secondary key is:
 		    (setq buffers (buffer-list))
 		    (switch-to-buffer (nth (1- (length buffers)) buffers))))
 		 ((hmouse-release-right-edge 'assist)
-		  (if (equal (buffer-name) (hypb:help-buf-name "Smart"))
+		  (if (string-match "Hyperbole Smart Keys" (buffer-name))
 		      (hkey-help-hide)
 		    (hkey-summarize 'current-window)))
-		 (t (run-hooks 'assist-key-modeline-hook))))
+		 (t (funcall assist-key-modeline-function))))
 	  (t (hmouse-modeline-resize-window 'assist)))))
 
 (defun hmouse-modeline-click (&optional assist-flag)
@@ -406,10 +631,10 @@ Optional ASSIST-FLAG non-nil means test for Assist Key click instead."
 
 (defun hmouse-modeline-depress ()
   "Returns non-nil if Action Key was depressed on a window mode line.
-If free variable 'assist-flag' is non-nil, uses Assist Key."
+If free variable `assist-flag' is non-nil, uses Assist Key."
   (let ((args (if assist-flag assist-key-depress-args
 		action-key-depress-args)))
-    (if (and hyperb:window-system args)
+    (if (and (hyperb:window-system) args)
 	(if (fboundp 'event-over-modeline-p)
 	    (event-over-modeline-p args)
 	  (let* ((w (smart-window-of-coords args))
@@ -425,7 +650,7 @@ If free variable 'assist-flag' is non-nil, uses Assist Key."
 Optional non-nil ASSIST-FLAG means test release of Assist Key instead."
   (let ((args (if assist-flag assist-key-release-args
 		action-key-release-args)))
-    (if (and hyperb:window-system args)
+    (if (and (hyperb:window-system) args)
 	(if (fboundp 'event-over-modeline-p)
 	    (event-over-modeline-p args)
 	  (let* ((w (smart-window-of-coords args))
@@ -438,8 +663,8 @@ Optional non-nil ASSIST-FLAG means test release of Assist Key instead."
 Resize amount depends upon the vertical difference between press and release
 of the Action Key.  Optional arg ASSIST-FLAG non-nil means use values from
 Assist Key instead."
-  (cond ((not hyperb:window-system) nil)
-	((and hyperb:xemacs-p (not (fboundp 'window-edges)))
+  (cond ((not (hyperb:window-system)) nil)
+	((and (featurep 'xemacs) (not (fboundp 'window-edges)))
 	 (error "Drag from a mode-line with button1 to resize windows."))
 	(t (let* ((owind (selected-window))
 		  (window (smart-window-of-coords
@@ -481,7 +706,7 @@ Assist Key instead."
 
 (defun hmouse-release-left-edge (&optional assist-flag)
   "Returns non-nil if last Action Key release was at left window edge.
-'hmouse-edge-sensitivity' value determines how near to actual edge the
+`hmouse-edge-sensitivity' value determines how near to actual edge the
 release must be."
   (let ((args (if assist-flag assist-key-release-args
 		 action-key-release-args))
@@ -493,13 +718,13 @@ release must be."
 	      window-left 0)
       (setq window-left (car (window-edges))
 	    last-release-x (and args (hmouse-x-coord args))))
-
     (and last-release-x (< (- last-release-x window-left)
-			   hmouse-edge-sensitivity))))
+			   hmouse-edge-sensitivity)
+	 (>= (- last-release-x window-left) 0))))
 
 (defun hmouse-release-right-edge (&optional assist-flag)
   "Returns non-nil if last Action Key release was at right window edge.
-'hmouse-edge-sensitivity' value determines how near to actual edge the
+`hmouse-edge-sensitivity' value determines how near to actual edge the
 release must be."
   (let ((args (if assist-flag assist-key-release-args
 		 action-key-release-args))
@@ -520,22 +745,22 @@ release must be."
 Resize amount depends upon the horizontal difference between press and release
 of the Action Key.  Optional arg ASSIST-FLAG non-nil means use values from
 Assist Key instead."
-  (cond (hyperb:xemacs-p
+  (cond ((featurep 'xemacs)
 	 ;; Depress events in scrollbars or in non-text area of buffer are
 	 ;; not visible or identifiable at the Lisp-level, so always return
 	 ;; nil.
 	 nil)
-	(hyperb:window-system
+	((hyperb:window-system)
 	 (let* ((owind (selected-window))
 		(window (smart-window-of-coords
 			 (if assist-flag assist-key-depress-args
 			   action-key-depress-args)))
-		(side-ln (and window (1- (nth 2 (window-edges window)))))
+		(right-side-ln (and window (1- (nth 2 (window-edges window)))))
 		(last-release-x
 		 (hmouse-x-coord
 		  (if assist-flag assist-key-release-args
 		    action-key-release-args)))
-		(shrink-amount (- side-ln last-release-x))
+		(shrink-amount (- right-side-ln last-release-x))
 		)
 	   ;; Restore position of point prior to Action Key release.
 	   (if action-key-release-prev-point
@@ -546,7 +771,7 @@ Assist Key instead."
 		       (goto-char (marker-position action-key-release-prev-point)))
 		   (set-buffer obuf))))
 	   (cond
-	    ((>= (+ side-ln 2) (frame-width))
+	    ((>= (+ right-side-ln 2) (frame-width))
 	     (error
 	      "(hmouse-resize-window-side): Can't change width of full frame width window."))
 	    ((< (length (hypb:window-list 'no-minibuf)) 2)
@@ -569,7 +794,7 @@ If optional arg ASSIST-FLAG is non-nil, uses Assist Key."
 	 (w2-buf (and w2 (window-buffer w2)))
 	 )
     (or (and w1 w2)
-	(error "(hmouse-swap-buffers): Last depress or release not within a window."))
+	(error "(hmouse-swap-buffers): Last depress or release was not within a window."))
     ;; Swap window buffers.
     (set-window-buffer w1 w2-buf)
     (set-window-buffer w2 w1-buf)))
@@ -587,7 +812,7 @@ If optional arg ASSIST-FLAG is non-nil, uses Assist Key."
 	 (w2-height (and w2 (window-height w2)))
 	 )
     (or (and w1 w2)
-	(error "(hmouse-swap-windows): Last depress or release not within a window."))
+	(error "(hmouse-swap-windows): Last depress or release was not within a window."))
     (unwind-protect
 	(progn
 	  (select-window w1)
@@ -607,53 +832,47 @@ If optional arg ASSIST-FLAG is non-nil, uses Assist Key."
       )))
 
 (defun hmouse-x-coord (args)
-  "Returns x coordinate in chars from window system dependent ARGS."
-  (let ((x (eval (cdr (assoc hyperb:window-system
-			     '(("emacs19" . (if (eventp args)
+  "Returns x coordinate in characters from window system dependent ARGS."
+  (let ((x (if (markerp args)
+	       (save-excursion
+		 (hypb:goto-marker args)
+		 (current-column))
+	     (eval (cdr (assoc (hyperb:window-system)
+			       '(("emacs" . (if (eventp args)
 						(+ (car (posn-col-row
 							 (event-start args)))
 						   (nth 0 (window-edges
-							   (car
-							    (car (cdr args))
-							    ))))
+							   (car (cadr args)))))
 					      (car args)))
-			       ("xemacs" .  (if (eventp args)
-						(event-x args)
-					      (car args)))
-			       ("xterm"  .  (car args))
-			       ("epoch"  .  (nth 0 args))   ;; Epoch V4
-			       ("sun"    .  (nth 1 args))
-			       ("next"   .  (nth 1 args))
-			       ("apollo" .  (car args))
-			       ))))))
+				 ("xemacs" .  (if (eventp args)
+						  (event-x args)
+						(car args)))
+				 ("xterm"  .  (car args))
+				 ("next"   .  (nth 1 args))
+				 )))))))
     (if (integerp x) x (error "(hmouse-x-coord): invalid X coord: %s" x))))
 
 (defun hmouse-y-coord (args)
   "Returns y coordinate in frame lines from window system dependent ARGS."
-  (let ((y (eval (cdr (assoc hyperb:window-system
-			     '(("emacs19" . (if (eventp args)
-						(+ (cdr (posn-col-row
-							 (event-start args)))
-						   (nth 1 (window-edges
-							   (car
-							    (car (cdr args))
-							    ))))
-					      (cdr args)))
+  (let ((y (eval (cdr (assoc (hyperb:window-system)
+			     '(("emacs" . (if (eventp args)
+					      (+ (cdr (posn-col-row
+						       (event-start args)))
+						 (nth 1 (window-edges
+							 (car (cadr args)))))
+					    (cdr args)))
 			       ("xemacs" .  (if (eventp args)
 						(event-y args)
 					      (cdr args)))
 			       ("xterm"  .  (nth 1 args))
-			       ("epoch"  .  (nth 1 args))   ;; Epoch V4
-			       ("sun"    .  (nth 2 args))
 			       ("next"   .  (nth 2 args))
-			       ("apollo" .  (nth 1 args))
 			       ))))))
     (if (integerp y) y (error "(hmouse-y-coord): invalid Y coord: %s" y))))
 
 
-;;;
+;;; ************************************************************************
 ;;; Private variables
-;;;
+;;; ************************************************************************
 
 
 (provide 'hui-window)
