@@ -194,7 +194,7 @@ set to 1.  If it is > `hycontrol-maximum-units', it is set to
 		   ((eq e ?n) (set-frame-width nil (- (frame-width) arg)))
 		   ((eq e ?o) (setq w (selected-window)) (other-window arg) (if (eq w (selected-window)) (other-window 1)))
 		   ((eq e ?O) (setq w (selected-window)) (other-frame arg) (if (eq w (selected-window)) (other-frame 1)))
-		   ((eq e ?q) (throw 'done t))
+		   ((memq e (list ?q (aref (kbd "<escape>") 0))) (throw 'done t))
 		   ((eq e ?s) (set-frame-height nil (- (frame-height) arg)))
 		   ((eq e ?t) (throw 'done nil))
 		   ((eq e ?u) (unbury-buffer))
@@ -279,7 +279,7 @@ set to 1.  If it is > `hycontrol-maximum-units', it is set to
 		   ((eq e ?n) (shrink-window-horizontally arg))
 		   ((eq e ?o) (setq w (selected-window)) (other-window arg) (if (eq w (selected-window)) (other-window 1)))
 		   ((eq e ?O) (setq w (selected-window)) (other-frame arg) (if (eq w (selected-window)) (other-frame 1)))
-		   ((eq e ?q) (throw 'done t))
+		   ((memq e (list ?q (aref (kbd "<escape>") 0))) (throw 'done t))
 		   ((eq e ?s) (shrink-window arg))
 		   ((eq e ?t) (throw 'done nil))
 		   ((eq e ?u) (unbury-buffer))
